import processing.serial.*;
PFont font;
PrintWriter output;

int  _BAUD    = 115200;
float _EULER[] = new float[3];


void setup() 
{
  size(1000, 1000, P3D);  /// Set window size on three axes
  Serial serial;
  
  /// This loop waits until a serial device is physical connected
  while (true)
  {
    try
    {
      serial = new Serial(this, Serial.list()[0], _BAUD);
      break;
    } catch (Exception e)
    {
      print("No device present\n");
      delay(1000);
    }
  }
  serial.clear();
  serial.bufferUntil('\n');  /// With this, serialEvent() is automatically run when newline is received
  
  font = createFont("verdana.ttf", 32);
  
  /// File writer
  output = createWriter("output.txt");
}



void draw()  /// this runs forever
{  
  background(#ffffff);  /// white background
  
  /// OSD _EULER angles
  textFont(font, 20);
  fill(#000000);    /// text color
  text("phi    : " + degrees(_EULER[0]) + "\ntheta: " + degrees(_EULER[1]) + "\npsi    : " + degrees(_EULER[2]), 10, 50);
  

  /// comensation
  float X = -_EULER[1] + 3.14/2;
  float Y =  _EULER[0] + 3.14;
  float Z = -_EULER[2] + 3.14;

  
  translate(550, 600, 0);  /// Move object away from center of origin
  scale(3,3,3);            /// Scale all objects times 3

  rotateX( X );
  rotateY( Y );
  rotateZ( Z );
  buildWingShape();
}



void serialEvent(Serial serial)
{
  /// Read input until newline
  String input = serial.readStringUntil('\n');
  if (input == null || input.length() == 0)
  {
    print(input);
    return;
  }
    
  /// Split input data by tab and print resulting data when array is too small
  String tmp[] = split(input, "\t");
  if (tmp.length < 3)
  {
    print(input);
    return;
  }
    
  for (int i=0;i<3;i++)
  {
    _EULER[i] = float(tmp[i]);
  }
    
  /// Write angles to file
  output.print(input);
}



void buildWingShape()
{
  //box(60, 10, 40);
  noStroke();
  beginShape(QUADS);
  
  /// X - right
  fill(#00ff00);
  vertex(-100, 0, -10);
  vertex(-100, 0, 10);
  vertex(-100, 40, 10);
  vertex(-100, 40, -10);
  
  /// X - left
  fill(#ff0000);
  vertex(100, 0, -10);
  vertex(100, 0, 10);
  vertex(100, 40, 10);
  vertex(100, 40, -10);
  
  /// Y front right
  fill(#00ffff);
  vertex(-100, 40, -10);
  vertex(-100, 40, 10);
  vertex(0, 120, 0);
  vertex(0, 120, 0);
  
  /// Y front left
  fill(#0000ff);
  vertex(100, 40, -10);
  vertex(100, 40, 10);
  vertex(0, 120, 0);
  vertex(0, 120, 0);
  
  /// Y rear right
  fill(#00ffff);
  vertex(-100, 0, -10);
  vertex(0, 40, -10);
  vertex(0, 40, 10);
  vertex(-100, 0, 10);

  /// Y rear left
  fill(#0000ff);
  vertex(100, 0, -10);
  vertex(0, 40, -10);
  vertex(0, 40, 10);
  vertex(100, 0, 10);
  
  /// Z bottom right
  fill(#ff0000);
  vertex(-100, 0, -10);
  vertex(0, 40, -10);
  vertex(0, 120, 0);
  vertex(-100, 40, -10);
  
  /// Z bottom left
  fill(#ff0000);
  vertex(100, 0, -10);
  vertex(0, 40, -10);
  vertex(0, 120, 0);
  vertex(100, 40, -10);
  
  /// Z top right
  fill(#ff0ff0);
  vertex(-100, 0, 10);
  vertex(0, 40, 10);
  vertex(0, 120, 0);
  vertex(-100, 40, 10);
  
  /// Z top left
  fill(#ff0ff0);
  vertex(100, 0, 10);
  vertex(0, 40, 10);
  vertex(0, 120, 0);
  vertex(100, 40, 10);
  
  
  endShape();
}



void drawAxes()
{
  /// Y
  stroke(255, 0, 0);
  line(-0, 0, 0, 500, 0, 0);
  
  /// X
  stroke(0, 255, 0);
  line(0, -0, 0, 0, 500, 0);
  
  /// Z
  stroke(0, 0, 255);
  line(0, 0, 0, 0, 0, 500);
}
