/*
generalized class for obtaining accelerations
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/

template <typename Driver> class Acceleration
{
	private:
		Driver _driver;
		
	private:
		int	_g;    		/// configured g value
		float	_bias[3];	/// holds calibration data from initiation routine
		float	_calibrationGainMin[3];
		float	_calibrationGainMax[3];
		float	_xAxis[3];	/// Holds acceleration in m/s^2 for three iterations. FIFO stack.
		float	_yAxis[3];	/// "
		float	_zAxis[3];	/// "
	
	
	public:
		void initialize(float xMin = -1, float xMax = 1, float yMin = -1, float yMax = 1, float zMin = -1, float zMax = 1);
		void bias(float x = 0.0, float y = 0.0, float z = 0.0);
		void raw();
		void measure();
		bool calibrate(bool calibrateXaxis = true, bool calibrateYaxis = true, bool calibrateZaxis = true, unsigned int threshold = 20, unsigned int samples = 128);
		float axis(int axis = 0, int index = 0);
		float x(int index = 0);
		float y(int index = 0);
		float z(int index = 0);
		bool setG(int g);
		bool setHz(float hz);
		float roll();
		float pitch();
};

