/*
BMP085 driver core
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#pragma once
#include <stdint.h>  /// uint8_t
#include "/home/v/Dropbox/Software/arduino/I2C_/I2C.h"

class BMP085
{
  private:
    I2C _i2c;
    #ifdef __MK20DX128__
      static constexpr float _temperatureCalibrate = -4.0;
      static constexpr short _OSS = 0; /// oversampling setting 0...3
    #else
      static const float _temperatureCalibrate = -4.0;
      static const short _OSS = 0; /// oversampling setting 0...3
    #endif

    long _pressureOffset;  /// Pascal at sea level
    long _UT;              /// always keep a UT storage such that we have a value while waiting 4.5ms
    long _UP;              /// same as UT but for pressure
    float _temperature;
    long _pressure;
    float _altitude;
    long _time;            /// initial values such that first measurement is not too early
    bool _state;           /// used as a state in an emulated interrupt    
    
    /// calibration values
    short _AC1;
    short _AC2;
    short _AC3;
    unsigned short _AC4;
    unsigned short _AC5;
    unsigned short _AC6;
    short _b1;           /// B1 is reserved by binary.h
    short _B2;
    long  _B5;
    short _MB;
    short _MC;
    short _MD;
  
  private:
    ///
    ///                 VARIABLE          HEX                  TYPE  DEFAULT_VALUE
    ///
    static const uint8_t   _DEVICE          = 0x77;
    
    /// calibration registers
    static const uint8_t   _AC1_MSB         = 0xAA;
    static const uint8_t   _AC1_LSB         = 0xAB;
    static const uint8_t   _AC2_MSB         = 0xAC;
    static const uint8_t   _AC2_LSB         = 0xAD;
    static const uint8_t   _AC3_MSB         = 0xAE;
    static const uint8_t   _AC3_LSB         = 0xAF;
    static const uint8_t   _AC4_MSB         = 0xB0;
    static const uint8_t   _AC4_LSB         = 0xB1;
    static const uint8_t   _AC5_MSB         = 0xB2;
    static const uint8_t   _AC5_LSB         = 0xB3;
    static const uint8_t   _AC6_MSB         = 0xB4;
    static const uint8_t   _AC6_LSB         = 0xB5;
    static const uint8_t   _B1_MSB          = 0xB6;
    static const uint8_t   _B1_LSB          = 0xB7;
    static const uint8_t   _B2_MSB          = 0xB8;
    static const uint8_t   _B2_LSB          = 0xB9;
    static const uint8_t   _MB_MSB          = 0xBA;
    static const uint8_t   _MB_LSB          = 0xBB;
    static const uint8_t   _MC_MSB          = 0xBC;
    static const uint8_t   _MC_LSB          = 0xBD;
    static const uint8_t   _MD_MSB          = 0xBE;
    static const uint8_t   _MD_LSB          = 0xBF;
    
    /// request register
    static const uint8_t  _REQUEST_REGISTER    = 0xF4;
    static const uint8_t  _REQUEST_MSB         = 0xF6;
    static const uint8_t  _REQUEST_LSB         = 0xF7;
    
    /// request values
    static const uint8_t  _TEMPERATURE_REQUEST = 0x2E;
    static const uint8_t  _PRESSURE_REQUEST    = 0x34 + (_OSS << 6);


  
  public:
    void initialize();
    void calibrate();
    
    void requestTemperature();
    long readTemperature();
    void calcTemperature();
    float temperature();
    
    void requestPressure();   
    long readPressure();
    void calcPressure();
    float pressure();
    
    void update();

};

