/*
L3G4200D driver core
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include <Arduino.h>
#include "L3G4200D.h"



bool L3G4200D::initialize()
{
  if (_i2c.read(_DEVICE, _WHO_AM_I, 1)[0] != 0xD3)	/// Sensor must have WHO_AM_I before power on
  {
    return false;
  }

  _i2c.write(_DEVICE, _CTRL_REG1, 0x8F);		/// Power on, enable all axes and put in 400 Hz mode
  _i2c.write(_DEVICE, _CTRL_REG2, 0x00);		/// Disable all filters
  _i2c.write(_DEVICE, _CTRL_REG3, 0x08);		///
  _i2c.write(_DEVICE, _CTRL_REG4, 0x30);		/// Set block mode, LSB, 2000 dps, disable self test and 4-wire I2C interface
  _i2c.write(_DEVICE, _CTRL_REG5, 0x00);		/// 
  
  return true;
}



int16_t * L3G4200D::measure()
{
  uint8_t * b = _i2c.read(_DEVICE, _OUT_X_L | (1 << 7), 6);
  
  this->_raw[0] = (b[1] << 8) | b[0];
  this->_raw[1] = (b[3] << 8) | b[2];
  this->_raw[2] = (b[5] << 8) | b[4];
  
  /// sort out orientation according to a right hand system
  this->_raw[1] *= -1;
  this->_raw[2] *= -1;
  
  return this->_raw;
}



int L3G4200D::temperature()
{
  /// in celsius, +273.15 to get in Kelvin
  return (int) _i2c.read(_DEVICE, _OUT_TEMP,1)[0];
}
