
#include "Wire.h"
#include "../sensors/I2C.cpp"

#include "../sensors/accelerometer/ADXL345_/ADXL345.cpp"
#include "../sensors/accelerometer/Acceleration.cpp"

#include "../sensors/gyro/L3G4200D_/L3G4200D.cpp"
#include "../sensors/gyro/Gyro.cpp"

#include "../sensors/magnetometer/HMC5883L_/HMC5883L.cpp"
#include "../sensors/magnetometer/Compass.cpp"

#include "AHRS.h"
AHRS ahrs(800, 0.1);

Acceleration<ADXL345>  acceleration;
Compass<HMC5883L>     compass;
Gyro<L3G4200D>         gyro;



void setup()
{
  Serial.begin(115200);
  #if defined(__MK20DX128__)
  pinMode(13,OUTPUT);
  int state = LOW;
  while (!Serial)
  {
    digitalWrite(13,state);
    state = (state==LOW?HIGH:LOW);
    delay(100);
  }
  digitalWrite(13,LOW);
  #endif
  Serial.println("Serial started");
  
  Wire.begin();

  acceleration.setHz(400);
  acceleration.setG(2); 
  acceleration.initialize(9.81/266, 9.81/254, 9.81/266, 9.81/257, 9.81/281, 9.81/244);
  while (!acceleration.calibrate(true,true,false,10))
  {
    Serial.println("ACCELEROMETER: re-calibrating");
    delay(1000);
  }
  Serial.println("Accelerometer calibrated");
  
  
  while (!gyro.initialize((float) 400 / 2000))  /// the gyro is default in sleep mode and must be set active, this fails sometimes
  {
    Serial.println("GYRO: init failed. Wrong address? Reattempt in 1 second.");
    delay(1000);
  }
  while (!gyro.calibrate(true,true,true,30))
  {
    Serial.println("GYRO: re-calibrating in 1 second.");
    delay(1000);
  }
  Serial.println("GYRO: calibrated");
  
  compass.initialize();
  compass.setGain(1);
  compass.continuous();
  delay(1000);
}


unsigned long time;
unsigned long pHz;
void loop()
{
  time = micros();
  ///
  /// 800 HZ LOOP
  ///
  if ( (time - pHz) >= 1250)
  {
    acceleration.measure();    
    gyro.measure();
    compass.measure();
    
    /// Altering gyro measurements due to noise!
    if ( abs(gyro.y(0) - gyro.y(1)) > 500)
    {
      gyro.setAxisValue(1,0,gyro.y(1));
    }
    if ( abs(gyro.z(0) - gyro.z(1)) > 500)
    {
      gyro.setAxisValue(2,0,gyro.z(1));
    }
    
    ahrs.MARG(acceleration.x(), acceleration.y(), acceleration.z(), radians(gyro.x()), radians(gyro.y()), radians(gyro.z()), compass.x(), compass.y(), compass.z());
//ahrs.ARG(acceleration.x(), acceleration.y(), acceleration.z(), radians(gyro.x()), radians(gyro.y()), radians(gyro.z()));
    
    Serial.print(ahrs.phi());
    Serial.print("\t");
    Serial.print(ahrs.theta());
    Serial.print("\t");
    Serial.print(ahrs.psi());
    Serial.print("\t");
    Serial.print(acceleration.x());
    Serial.print("\t");
    Serial.print(acceleration.y());
    Serial.print("\t");
    Serial.print(acceleration.z());
    Serial.print("\t");
    Serial.print(gyro.x());
    Serial.print("\t");
    Serial.print(gyro.y());
    Serial.print("\t");
    Serial.print(gyro.z());
    Serial.println();
    
    pHz = time;
  }
}
