/*
Generalised class to handle navigation tasks and GPS receiver commnication. Thus communication with NMEA parser (TinyGPS)
Written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "TinyGPS.h"

class Navigation
{
  public:
        struct Geodetic
        {
            float latitude;
            float longitude;
            float altitude;
        };
        
  private:
    TinyGPS _GPS;
    static const long _r_e = 6378137;  /// equatorial radius of ellipsoid (semimajor axis)
    static const long _r_p = 6356752;  /// polar axis radius of ellipsoid (semimajor axis)


  /// waypoint coordinates in geodic format
  private:
    int _waypointCounter;
    int _numberOfWaypoints;
    static const int _maxWaypoints = 200;
    
  /// geodic coordinates
  private:
    Geodetic _origin;                        /// Coordinate of where we started our travel
    Geodetic _waypoint[_maxWaypoints];       /// All stored waypoints
    Geodetic _waypointTo;      /// Coordinates where we are to dravel
    Geodetic _location;        /// Last known position
    unsigned long _age;     /// number of milliseconds since we updated current position
    
  
  public:
    bool begin(int baudRate = 9600);
    float course();
    int satellites();
    bool update();
    bool next(int threshold = 10);
    void geodetic2cartesian(long latitude, long longitude, long height);
    void status();
    bool loadNextWaypoint();
    void loadWaypoints(float * latitude, float * longitude, float * height);
    void origin();
    void setOrigin(float latitude, float longitude, float altitude);
    float latitude();
    float longitude();
    float altitude();
    float latitudeTo();
    float longitudeTo();
    float altitudeTo();
    void setHome();
    void addWaypoint(float latitude, float longitude, float altitude);
    void clearWaypoints();
    void home();
    bool toHome();
    float distance(float latitude, float longitude, float latitudeTo, float longitudeTo);
};
