#include <Arduino.h>  /// attachInterrupt(), detachInterrupt()
#include "PWM.h"


/// add a pin you want to monitor
void PWM::add(int pin, functionPointer customFunction, int initialValue, int filterLowPass)
{
  _signal             = initialValue;	/// initialize a signal value
  _interruptStartTime = micros();	/// initialize start time such that high pulse does not become too big!
  _filterLowPass      = filterLowPass;
  pinMode(pin, INPUT);
  attachInterrupt(pin, customFunction, RISING);
}


/// remove pin monitoring
void PWM::remove(int pin)
{
  detachInterrupt(pin);
}


/// triggered when a pin changes from low to high
void PWM::handleRising(int pin, functionPointer customFunction)
{
  _interruptStartTime = micros();                 /// Store start time to be used in fall()
  detachInterrupt(pin);                           /// Detach..
  attachInterrupt(pin, customFunction, FALLING);  /// ..and set it to falling
}


/// triggered when a pin changes from high to low
void PWM::handleFalling(int pin, functionPointer customFunction)
{
  int pulse = micros() - _interruptStartTime;  /// calculate signal pulse
  
  detachInterrupt(pin);
  attachInterrupt(pin, customFunction, RISING);
  
  /// if the pulse is outside expected area, ignore it
  if (pulse > PWM::_maxPulseWidth || pulse < PWM::_minPulseWidth)
  {
    return;
  }

  /// simple optional filter for large changes
  if (_filterLowPass != 0 && abs(pulse - _signal) > _filterLowPass)
  {
    return;
  }
  
  _signal = pulse;
}


/// Return "high" PWM pulse time which has been recorded by the receiver.
int PWM::signal()
{
  return _signal;
}


/// RC systems uses angles from 0 to 180, provide this
int PWM::angle()
{
  return map(_signal, this->_minPulseWidth, this->_maxPulseWidth, this->_minAngle, this->_maxAngle);
}


/// return the smallest allowed pulse
int PWM::minPulseWidth()
{
  return _minPulseWidth;
}


/// return the highest allowed pulse
int PWM::maxPulseWidth()
{
  return _maxPulseWidth;
}
