/// Serial configuration
static const unsigned int _SERIAL_BAUD      = 115200;
static const unsigned int _WIRELESS_BAUD    = 9600;
static const unsigned int _GPS_BAUD         = 9600;

/// Misc
static const float _GAIN = 0.3;
static const unsigned int _HERTZ  = 400;
static const unsigned int _MAX_G  = 2;
static const unsigned int _NAVIGATION_THRESHOLD = 10;  /// in meters
static const unsigned int _ACCELEROMETER_NOISE = 10;
static const unsigned int _GYRO_NOISE = 30;

/// Receiver
static const int _PIN_THROTTLE = 2;  /// Blue
static const int _PIN_AILERON  = 3;  /// Yellow
static const int _PIN_ELEVATOR = 4;  /// Purple
static const int _PIN_MODE     = 5;  /// White


/// The following are threshold values for mode selection based on incomming pulse length on mode channel
const int _SIGNAL_MANUAL_MODE    = 1300;
const int _SIGNAL_AIDED_MODE     = 1600;
const int _SIGNAL_WP_MODE        = 1800;


/// Acutators
const int _PIN_MOTOR = 22;  /// Electronic speed controller for motor
const int _PIN_LEFT  = 21;  /// Left rudder servo
const int _PIN_RIGHT = 20;  /// Right rudder servo


/// PID values
const double _SURGE_P = 1;
const double _SURGE_I = 0;
const double _SURGE_D = 0;

const double _PITCH_P = 0.9;
const double _PITCH_I = 0;
const double _PITCH_D = 0;

const double _ROLL_P = 0.9;
const double _ROLL_I = 0;
const double _ROLL_D = 0;

const double _YAW_P = 0.6;
const double _YAW_I = 0;
const double _YAW_D = 0;


/// Actuator constraints, NOT closed loop constraints
const double _MAX_ROLL_ANGLE  = 45;  /// degrees
const double _MAX_PITCH_ANGLE = 60;  /// degrees
const double _MIN_PITCH_ANGLE = -30; /// degrees
const double _MAX_YAW_ANGLE   = 30;  /// degrees


/// Vehicle identificator
char * _IDENTITY = "F#5";  /// Can not have spaces in name!
bool _LAUNCHED = false;


///
/// The constants below are used for debugging purposes
///
//#define _DEBUG 1      /// Nothing yet
//#define _NMEA 1       /// Show raw GPS NMEA data
//#define _ERRORS 1     /// Show error variables in euler angles
//#define _IMU 1
//#define _STATUS 1
//#define _SIMULATOR 1  /// Show euler angles, used for visualsation
//#define _SIGNALS 1    /// Show parsed receiver signals
//#define _DISABLE_ACTUATOR 1
