/*
PID regulator
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "PID.h"

PID::PID()
{
  __P = 1;
  _I = 0;
  _D = 0;
  _windUp = 40;
  _integral = 0;
  _errorDot = 0;
}


/// This is not declared as constructor such that gains can be changed in real time
void PID::set(float proportional, float integral, float derivative, float windUp)
{
  __P = proportional;
  _I = integral;
  _D = derivative;
  
  _windUp = windUp;
}


/// Famous PID algorithm
float PID::stabilize(float desiredState, float state)
{
  float error = state - desiredState;
  
  _integral = _integral + error;
  
  if (error == 0)
  {
    _integral = 0;
  }
  else if (error > _windUp)  /// wind up
  {
    _integral = 0;
  }
  
  float derivative = error - _errorDot;
  
  _errorDot = error;
  
  return __P * error + _I * _integral + _D * derivative;
}
