/*
ADXL345 driver core
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "ADXL345.h"

void ADXL345::initialize()
{
  _i2c.write(_DEVICE, _DATA_FORMAT, 0x08);  /// Put sensor in full resolution and +/- 2g
  _i2c.write(_DEVICE, _POWER_CTL, 0x0B);    /// Put sensor in measure mode (turn on)
}


/// Get accelerations from accelerometer
int16_t * ADXL345::measure()
{ 
  /// there are 6 registers for acceleration data, we start by reading from register DATAX0 and pluss 5
  uint8_t * b = _i2c.read(_DEVICE, _DATAX0, 6);
  
  this->_raw[0] = (b[1] << 8) | b[0];
  this->_raw[1] = (b[3] << 8) | b[2];
  this->_raw[2] = (b[5] << 8) | b[4];
  
  this->_raw[0] *= -1;
  
  return this->_raw;
}


/// Set desired +/- g
bool ADXL345::setG(int g)
{
  /// values from datasheet
  int gravity[4]   = {2, 4, 8, 16};
  uint8_t codes[4] = {00, 01, 10, 11};
  
  bool ok = false;
  int i;
  for (i=3; i >= 0; i--)
  {
    if (gravity[i] == g)
    {
      ok = true;
      break;
    }
  }

  if (!ok)
  {
    return false;
  }
  
  /// write the desired rate to sensor
  _i2c.write(_DEVICE, _DATA_FORMAT, codes[i]);
  
  return true;
}


/// Set desired frequency update
bool ADXL345::setHz(float hz)
{
  /// values from the datasheet
  float hertz[16]     = {3200, 1600, 800, 400, 200, 100, 50, 25, 12.5, 6.25, 3.13, 1.56, 0.78, 0.39, 0.20, 0.10};
  float bandwidth[16] = {1600, 800, 400, 200, 100, 50, 25, 12.5, 6.25, 3.13, 1.56, 0.78, 0.39, 0.20, 0.10, 0.05};
  uint8_t codes[16]   = {1111, 1110, 1101, 1100, 1011, 1010, 1001, 1000, 0111, 0110, 0101, 0100, 0011, 0010, 0001, 0000};
  
  bool ok = false;
  int i;
  for (i=15; i >= 0; i--)
  {
    if (hertz[i] == hz)
    {
      ok = true;
      break;
    }
  }
  
  if (!ok)
  {
    return false;
  }
  
  /// write the desired rate to sensor
  _i2c.write(_DEVICE, _BW_RATE, codes[i]);
  
  return true;
}
