/*
generalized class for obtaining accelerations
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/

#include <stdint.h>  /// uint8_t
#include "/home/v/Dropbox/Software/arduino/I2C_/I2C.h"


class ADXL345
{
  private:
    ///
    ///                 VARIABLE          HEX  DECIMAL  TYPE    Description
    ///
    static const uint8_t        _DEVICE          = 0x53;
    //static const uint16_t	_DEVID	        = 0x00;	//0	R 	Device ID
    //static const uint16_t     Reserved	0x01 to 0x1C	1 to 28		Reserved; do not access
    static const uint8_t	_THRESH_TAP	= 0x1D;	//29	R/W 	Tap threshold
    static const uint8_t        _OFSX	        = 0x1E;	//30	R/W 	X-axis offset
    static const uint8_t	_OFSY	        = 0x1F;	//31	R/W 	Y-axis offset
    static const uint8_t	_OFSZ	        = 0x20;	//32	R/W 	Z-axis offset
    static const uint8_t	_DUR	        = 0x21;	//33	R/W 	Tap duration
    static const uint8_t	_Latent	        = 0x22;	//34	R/W 	Tap latency
    static const uint8_t	_Window	        = 0x23;	//35	R/W 	Tap window
    static const uint8_t	_THRESH_ACT	= 0x24;	//36	R/W 	Activity threshold
    static const uint8_t	_THRESH_INACT	= 0x25;	//37	R/W 	Inactivity threshold
    static const uint8_t	_TIME_INACT	= 0x26;	//38	R/W 	Inactivity time
    static const uint8_t	_ACT_INACT_CTL	= 0x27;	//39	R/W 	Axis enable control for activity and inactivity detection
    static const uint8_t	_THRESH_FF	= 0x28;	//40	R/W 	Free-fall threshold
    static const uint8_t	_TIME_FF	        = 0x29;	//41	R/W 	Free-fall time
    static const uint8_t	_TAP_AXES	= 0x2A;	//42	R/W 	Axis control for single tap/double tap
    static const uint8_t	_ACT_TAP_STATUS	= 0x2B;	//43	R 	Source of single tap/double tap
    static const uint8_t	_BW_RATE	        = 0x2C;	//44	R/W 	Data rate and power mode control
    static const uint8_t	_POWER_CTL	= 0x2D;	//45	R/W 	Power-saving features control
    static const uint8_t	_BYTE_ENABLE	= 0x2E;	//46	R/W 	uint8_terrupt enable control
    static const uint8_t	_BYTE_MAP	        = 0x2F;	//47	R/W 	uint8_terrupt mapping control
    static const uint8_t	_BYTE_SOURCE	= 0x30;	//48	R 	Source of uint8_terrupts
    static const uint8_t	_DATA_FORMAT	= 0x31;	//49	R/W 	Data format control
    static const uint8_t	_DATAX0	        = 0x32;	//50	R 	X-Axis Data 0
    static const uint8_t	_DATAX1	        = 0x33;	//51	R 	X-Axis Data 1
    static const uint8_t	_DATAY0	        = 0x34;	//52	R 	Y-Axis Data 0
    static const uint8_t	_DATAY1	        = 0x35;	//53	R 	Y-Axis Data 1
    static const uint8_t	_DATAZ0	        = 0x36;	//54	R 	Z-Axis Data 0
    static const uint8_t	_DATAZ1	        = 0x37;	//55	R 	Z-Axis Data 1
    static const uint8_t	_FIFO_CTL	= 0x38;	//56	R/W 	FIFO control
    static const uint8_t	_FIFO_STATUS	= 0x39;	//57	R 	FIFO status

  private:
    I2C     _i2c;
    int16_t _raw[3];
    
  public:
    void initialize();
    int16_t * measure();
    bool setG(int g);
    bool setHz(float hz);
};

