/*
BMA180 driver core
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "BMA180.h"

void BMA180::initialize()
{
  _i2c.write(this->_DEVICE, this->_RESET, 0xB6);       /// restart sensor
  _i2c.write(this->_DEVICE, this->_CTRL_REG0, 0x10);   /// turn on
}


/// Get accelerations from accelerometer
int16_t * BMA180::measure()
{ 
  /// there are 6 registers for acceleration data, we start by reading from register ACC_X_LSB and pluss 5
  uint8_t * b = _i2c.read(this->_DEVICE, this->_ACC_X_LSB, 6);
  
  this->_raw[0] = ( ( b[0] | b[1]<<8)>>2);
  this->_raw[1] = ( ( b[2] | b[3]<<8)>>2);
  this->_raw[2] = ( ( b[4] | b[5]<<8)>>2);
  
  /// set correct orientation corresponded for a right hand system, x forward, y right, z down
  this->_raw[1] *= -1;
  
  return this->_raw;
}


/// Set desired +/- g
bool BMA180::setG(int g)
{
  /// values from datasheet
  float gravity[7] = {1, 1.5, 2, 3, 4, 8, 16};
  uint8_t codes[7] = {000, 001, 010, 011, 100, 101, 110};

  
  bool ok = false;
  int i;
  for (i=6; i >= 0; i--)
  {
    if (gravity[i] == g)
    {
      ok = true;
      break;
    }
  }

  if (!ok)
  {
    return false;
  }
  
  /// write the desired rate to sensor
  _i2c.write(_DEVICE, _OFFSET_LSB1, codes[i]);  /// not sure if this is right, may have to be shifted
  
  return true;
}


/// Set desired frequency update
bool BMA180::setHz(float hz)
{
  /// values from the datasheet
  float hertz[8]     = {10, 20, 40, 75, 150, 300, 600, 1200};
  uint8_t codes[8]   = {0000, 0001, 0010, 0011, 0100, 0101, 0110, 0111};
  
  bool ok = false;
  int i;
  for (i=8; i >= 0; i--)
  {
    if (hertz[i] == hz)
    {
      ok = true;
      break;
    }
  }
  
  if (!ok)
  {
    return false;
  }
  
  /// write the desired rate to sensor
  _i2c.write(_DEVICE, _BW_TCS, codes[i]);  /// not sure if BW_TCS is corerct, may have to be shifted
  
  return true;
}
