/*
generalized class for obtaining gyro measurements
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#pragma once
#include <stdint.h>  /// uint8_t

template <typename Driver> class Altimeter
{
  private:
  	Driver  _driver;
  	int16_t _heightOffset;
	
	
	private:
		static const float _Lb = -0.0065;	/// [K/m]
		static const float _R = 8.31432;	/// [N m/mol K]
		static const float _g = 9.80665;	/// [m/s^2]
		static const float _M = 0.0289644;	/// [kg/mol]
		static const float _Patm = 101325;	/// [Pa]
  
  public:
    void initialize();
    void reset();
    float pressure();
    float temperature();
    float measure();
};

