/*
generalized class for obtaining accelerations
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/
#include "Arduino.h"  /// typedef of uint16_t


class Acceleration
{
  private:
    ///
    ///                 VARIABLE          HEX  DECIMAL  TYPE    Description
    ///
    static const uint16_t   DEVICE          = 0x53;
    //static const uint16_t	DEVID	        = 0x00;	//0	R 	Device ID
    //private: static const uint16_tReserved	0x01 to 0x1C	1 to 28		Reserved; do not access
    //static const uint16_t	THRESH_TAP	= 0x1D;	//29	R/W 	Tap threshold
    //static const uint16_t	OFSX	        = 0x1E;	//30	R/W 	X-axis offset
    //static const uint16_t	OFSY	        = 0x1F;	//31	R/W 	Y-axis offset
    //static const uint16_t	OFSZ	        = 0x20;	//32	R/W 	Z-axis offset
    //static const uint16_t	DUR	        = 0x21;	//33	R/W 	Tap duration
    //static const uint16_t	Latent	        = 0x22;	//34	R/W 	Tap latency
    //static const uint16_t	Window	        = 0x23;	//35	R/W 	Tap window
    //static const uint16_t	THRESH_ACT	= 0x24;	//36	R/W 	Activity threshold
    //static const uint8_t	THRESH_INACT	= 0x25;	//37	R/W 	Inactivity threshold
    //static const uint8_t	TIME_INACT	= 0x26;	//38	R/W 	Inactivity time
    //static const uint8_t	ACT_INACT_CTL	= 0x27;	//39	R/W 	Axis enable control for activity and inactivity detection
    //static const uint8_t	THRESH_FF	= 0x28;	//40	R/W 	Free-fall threshold
    //static const uint8_t	TIME_FF	        = 0x29;	//41	R/W 	Free-fall time
    //static const uint8_t	TAP_AXES	= 0x2A;	//42	R/W 	Axis control for single tap/double tap
    //static const uint8_t	ACT_TAP_STATUS	= 0x2B;	//43	R 	Source of single tap/double tap
    static const uint8_t	BW_RATE	        = 0x2C;	//44	R/W 	Data rate and power mode control
    static const uint8_t	POWER_CTL	= 0x2D;	//45	R/W 	Power-saving features control
    //static const uint8_t	uint8_t_ENABLE	= 0x2E;	//46	R/W 	uint8_terrupt enable control
    //static const uint8_t	uint8_t_MAP	        = 0x2F;	//47	R/W 	uint8_terrupt mapping control
    //static const uint8_t	uint8_t_SOURCE	= 0x30;	//48	R 	Source of uint8_terrupts
    static const uint8_t	DATA_FORMAT	= 0x31;	//49	R/W 	Data format control
    static const uint8_t	DATAX0	        = 0x32;	//50	R 	X-Axis Data 0
    //static const uint8_t	DATAX1	        = 0x33;	//51	R 	X-Axis Data 1
    //static const uint8_t	DATAY0	        = 0x34;	//52	R 	Y-Axis Data 0
    //static const uint8_t	DATAY1	        = 0x35;	//53	R 	Y-Axis Data 1
    //static const uint8_t	DATAZ0	        = 0x36;	//54	R 	Z-Axis Data 0
    //static const uint8_t	DATAZ1	        = 0x37;	//55	R 	Z-Axis Data 1
    //static const uint8_t	FIFO_CTL	= 0x38;	//56	R/W 	FIFO control
    //static const uint8_t	FIFO_STATUS	= 0x39;	//57	R 	FIFO status
    ///
    ///                 BW        HZ
    ///
    static const uint8_t	BW_1600	= 1111;
    static const uint8_t	BW_800	= 1110;
    static const uint8_t	BW_400	= 1101;
    static const uint8_t	BW_200	= 1100;
    static const uint8_t	BW_100	= 1011;
    static const uint8_t	BW_50	= 1010;
    static const uint8_t	BW_25	= 1001;
    static const uint8_t	BW_125	= 1000;
    static const uint8_t	BW_625	= 0111;
    static const uint8_t	BW_313	= 0110;
    static const uint8_t	BW_156	= 0101;
    static const uint8_t	BW_078	= 0100;
    static const uint8_t	BW_039	= 0011;
    static const uint8_t	BW_020	= 0010;
    static const uint8_t	BW_010	= 0001;
    static const uint8_t	BW_005	= 0000;

  private:
    uint8_t  buffer[6];
    int   maxOutput;               /// maximum output from a single axis
    double bias[3];      /// holds calibration data from initiation routine
    double calibrationGainMin[3];
    double calibrationGainMax[3];
    double xAxis[3];  /// Holds acceleration in m/s^2 for tre iterations. FIFO stack.
    double yAxis[3];  /// "
    double zAxis[3];  /// "

  
  public:
    int16_t raw[3];
    void begin();
    void write(uint8_t device, uint8_t address, uint8_t code);
    uint8_t * read(uint8_t device, uint8_t address, int numberOfregisters = 1);
    void get();
    void getRaw();
    bool calibrate(bool calibrateXaxis = true, bool calibrateYaxis = true, bool calibrateZaxis = true);
    double axis(int axis = 0, int index = 0);
    double x(int index = 0);
    double y(int index = 0);
    double z(int index = 0);
    double roll();
    double pitch();
};

