/*
generalized class for obtaining gyro measurements
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/

#include <Arduino.h>

class Gyro
{
  private:
    ///
    ///                 VARIABLE          HEX                  TYPE  DEFAULT_VALUE
    ///
    static const uint8_t   DEVICE          = (0xD0 >> 1);
    //static const uint8_t	Reserved	= 00-0E;       //	-    -
    //static const uint8_t	Reserved	= 10-1F;       //	-    -
    static const uint8_t	WHO_AM_I	= 0x0F;        //	r    11010011
    static const uint8_t	CTRL_REG1	= 0x20;        //	rw   00000111
    static const uint8_t	CTRL_REG2	= 0x21;        //	rw   0x00
    static const uint8_t	CTRL_REG3	= 0x22;        //	rw   0x00
    static const uint8_t	CTRL_REG4	= 0x23;        //	rw   0x00
    static const uint8_t	CTRL_REG5	= 0x24;        //	rw   0x00
    static const uint8_t	REFERENCE	= 0x25;        //	rw   0x00
    static const uint8_t	OUT_TEMP	= 0x26;        //	r    output
    static const uint8_t	STATUS_REG	= 0x27;        //	r    output
    static const uint8_t	OUT_X_L	        = 0x28;        //	r    output
    static const uint8_t	OUT_X_H	        = 0x29;        //	r    output
    static const uint8_t	OUT_Y_L	        = 0x2A;        //	r    output
    static const uint8_t	OUT_Y_H	        = 0x2B;        //	r    output
    static const uint8_t	OUT_Z_L	        = 0x2C;        //	r    output
    static const uint8_t	OUT_Z_H	        = 0x2D;        //	r    output
    static const uint8_t	FIFO_CTRL_REG	= 0x02;        //	rw   0x00
    static const uint8_t	FIFO_SRC_REG	= 0x2F;        //	r
    static const uint8_t	INT1_CFG	= 0x30;        //	rw   0x00
    static const uint8_t	INT1_SRC	= 0x31;        //	r
    static const uint8_t	INT1_TSH_XH	= 0x32;        //	rw   0x00
    static const uint8_t	INT1_TSH_XL	= 0x33;        //	rw   0x00
    static const uint8_t	INT1_TSH_YH	= 0x34;        //	rw   0x00
    static const uint8_t	INT1_TSH_YL	= 0x35;        //	rw   0x00
    static const uint8_t	INT1_TSH_ZH	= 0x36;        //	rw   0x00
    static const uint8_t	INT1_TSH_ZL	= 0x37;        //	rw   0x00
    static const uint8_t	INT1_DURATION	= 0x38;        //	rw   0x00



  private:
    #ifdef __MK20DX128__
    static constexpr float _gain = 0.25;
    #else
    static const float _gain = 0.25;
    #endif
    uint8_t buffer[6];
    int8_t bias[3];
    int16_t raw[3];
    int16_t xAxis[3];            /// -ready- values to be used generated by get()
    int16_t yAxis[3];
    int16_t zAxis[3];
    float xAvg[10];
    float yAvg[10];
    float zAvg[10];

  
  public:
    bool begin();
    bool calibrate(bool calibrateXaxis = true, bool calibrateYaxis = true, bool calibrateZaxis = true);
    void write(uint8_t device, uint8_t address, uint8_t code);
    uint8_t * read(uint8_t device, uint8_t address,int numberOfregisters = 1);
    void getRaw();
    void get();
    void getAverage();
    float axis(int axis = 0,int index = 0);
    float x(int index = 0);
    float y(int index = 0);
    float z(int index = 0);
    float heading();
    float latitude();
    int temperature();
};

