/*
generalized class for obtaining gyro measurements
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/

template <typename Driver> class Gyro
{
  private:
    Driver _driver;
    
  private:
    float _gain;
    int8_t _bias[3];
    int16_t _xAxis[3];
    int16_t _yAxis[3];
    int16_t _zAxis[3];
    float _xAvg[10];
    float _yAvg[10];
    float _zAvg[10];

  
  public:
    bool initialize(float gain = 1);
    void bias(float xBias, float yBias, float zBias);
    bool calibrate(bool calibrateXaxis = true, bool calibrateYaxis = true, bool calibrateZaxis = true, unsigned int threshold = 50, unsigned int samples = 128);
    void measure();
    void getAverage(int num = 10);
    float axis(int axis = 0,int index = 0);
    void setAxisValue(int axis, int index, double value);
    float x(int index = 0);
    float y(int index = 0);
    float z(int index = 0);
    float heading();
    float latitude();
    int temperature();
};

