/*
ITG3205 driver core
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com)
*/

#include "ITG3205.h"

bool ITG3205::initialize()
{
  _i2c.write(this->_DEVICE, this->_PWR_MGM, 0x00);  /// restart
  sampleRateDevider(7);
  _i2c.write(this->_DEVICE, this->_DLPF_FS, 0x1E);  /// +/- 2000 dgrs/sec, 1KHz, 1E, 19
  _i2c.write(this->_DEVICE, this->_INT_CFG, 0x00);
}

int16_t * ITG3205::measure()
{
  uint8_t * b = _i2c.read(this->_DEVICE, this->_GYRO_XOUT_H, 6);
  
  /// put data together
  this->_raw[0] = (b[0] << 8) | b[1];
  this->_raw[1] = (b[2] << 8) | b[3];
  this->_raw[2] = (b[4] << 8) | b[5];
  
  /// set orientation for a right hand system where x is forward, y left and z up
  this->_raw[0] *= -1;
  this->_raw[2] *= -1;
  
  
  return this->_raw;
}

int ITG3205::temperature()
{
  uint8_t * b = _i2c.read(this->_DEVICE, this->_TEMP_OUT_H, 2);
  
  /// in celsius, add 273.15 to get in Kelvin
  return (int) ( (b[0] << 8) | b[1] );
}

void ITG3205::sampleRateDevider(int rate)
{
  if (rate < 0)   rate = 0;
  if (rate > 255) rate = 255;
  
  _i2c.write(this->_DEVICE, this->_SMPLRT_DIV, (uint8_t) rate);
}


