/*
generalized class for magnetic flux from poles using a arbitrarily 3-axis magnetometer
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "Compass.h"


template <typename Driver> void Compass<Driver>::initialize()
{ 
  _driver.initialize();
}



template <typename Driver> void Compass<Driver>::measure()
{
  int16_t * buffer = _driver.measure();
  
  this->_axis[0] = buffer[0];
  this->_axis[1] = buffer[1];
  this->_axis[2] = buffer[2];
}



template <typename Driver> float Compass<Driver>::x()
{
  return this->_axis[0];
}
template <typename Driver> float Compass<Driver>::y()
{
  return this->_axis[1];
}
template <typename Driver> float Compass<Driver>::z()
{
  return this->_axis[2];
}

template <typename Driver> void Compass<Driver>::setGain(int index)
{
	_driver.setGain(index);
}

/// put compass in continuous measuring mode
template <typename Driver> void Compass<Driver>::continuous()
{
  _driver.continuous();
}

/// put compass in single mode
template <typename Driver> void Compass<Driver>::periodic()
{
	_driver.periodic();
}

/// put compass in idle mode
template <typename Driver> void Compass<Driver>::idle()
{
	_driver.idle();
}


template <typename Driver> float Compass<Driver>::roll(float declination)  /// phi
{
  float angle = ( atan2(y(),z()) + declination)*(180/M_PI);	/// convert to degrees
  if (angle < 0.0)
  {
    angle += 360;
  }
  
  return angle;  /// Use heading*M_PI/180 to obtain radians
}

template <typename Driver> float Compass<Driver>::pitch(float declination)  /// theta
{
  float angle = ( atan2(x(),z()) + declination)*(180/M_PI);	/// convert to degrees
  if (angle < 0.0)
  {
    angle += 360;
  }
  
  return angle;
}


template <typename Driver> float Compass<Driver>::heading(float declination)  /// psi
{
  float heading = ( atan2(y(), x()) + declination)*(180/M_PI);	/// convert to degrees
  
  /// correct for when signs are reversed.
  if (heading < 0.0)
  {
    heading += 360;
  }
  /// check for wrap due to addition of declination.
  else if (heading > 360.0)
  {
    heading -= 360;
  }

  return heading;
}



template <typename Driver> bool Compass<Driver>::selfTest()
{
	if (_driver.selfTest())
	{
		return true;
	}
	return false;
}



template <typename Driver> bool Compass<Driver>::calibrate()
{
	/// not implemented
  return true;
}