/*
Keep track of registered vehicles
Written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "Vehicle.hpp"
#include "JsBridge.hpp"
#include "QString"
#include <QObject>
#include <QDebug>
#include <QDir>


Vehicle::Vehicle(QObject * parent) : QObject(parent)
{
    _parent = parent;
}



Vehicle::~Vehicle()
{
    //delete _parent;
}



void Vehicle::add(QString id, double latitude, double longitude, double altitude)
{
    if (id.length() == 0)
    {
        return;
    }
    _vehicles.push_back(id);

    QStringList js;
    js << "var position = new google.maps.LatLng(" + QString::number(latitude, 'f', 8) + ", " + QString::number(longitude, 'f', 8) + ");";
    js << "var image = 'file://"+QDir::currentPath()+"/icon-airplane.png';";
    js << "var title = '" + id + "';";
    js << "_info['"+id+"'] = '"+formatPopup(id, latitude, longitude)+"';";
    js << "var marker = new google.maps.Marker({id: '"+id+"', position: position, map: _map, icon: image, title: title });";
    js << "_vehicles.push(marker);";
    js << "var infowindow = new google.maps.InfoWindow();";
    js << "google.maps.event.addListener(marker,'click',function() { infowindow.setContent(_info['"+id+"']);infowindow.open(_map, marker); });";

    /// send data to parent function
    JsBridge * parent = qobject_cast<JsBridge*>(_parent);
    parent->evaluateJs(js.join("\n"));

    parent->setStatus("New drone connected");
}



QStringList Vehicle::list()
{
    return _vehicles;
}



void Vehicle::move(QString id, double latitude, double longitude, double altitude)
{
    QStringList js;
    js << "var position = new google.maps.LatLng(" + QString::number(latitude, 'f', 8) + ", " + QString::number(longitude, 'f', 8) + ");";
    js << "for (var m = 0; m < vehicles.length; m++) { if (vehicles[m].id == '" + id + "') { vehicles[m].setPosition(position); } }";
    js << "_info['"+id+"'] = '"+formatPopup(id, latitude, longitude)+"';";

    /// send data to parent function
    JsBridge * parent = qobject_cast<JsBridge*>(_parent);
    parent->evaluateJs(js.join("\n"));
}



bool Vehicle::exists(QString id)
{
    for (int i=0; i < _vehicles.length(); i++ )
    {
        if (_vehicles.at(i) == id)
        {
            return true;
        }
    }

    return false;
}



QString Vehicle::formatPopup(QString id, double latitude, double longitude)
{
    QStringList code;
    code << "<strong>"+id+"</strong><br />Latitude: &nbsp; &nbsp;"+QString::number(latitude,'f')+"<br />";
    code << "Longitude: "+QString::number(longitude,'f')+"<br /><br />";
    code << "<a href=\"javascript:void(0);\" onclick=\"qt.returnHome(\\'"+id+"\\');\">Return home</a>";

    return code.join("");
}


/// Clear all vehicles
void Vehicle::clear()
{
    _vehicles.clear();
}



void Vehicle::remove(QString id)
{
    int i;
    for (i=0; i < _vehicles.length(); i++)
    {
        if (_vehicles.at(i) == id)
        {
            break;
        }
    }

    _vehicles.removeAt(i);
}