/*
Search And Rescue Control Center
Written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during winter 2013
*/

#include "mainwindow.hpp"
#include "ui_mainwindow.h"
#include "JsBridge.hpp"
#include "Serial.hpp"
#include <QtWebKit/QWebView>
#include <QFile>
#include <QTextStream>
#include <QWebFrame>
#include <QStringList>
#include <QLabel>
#include <QDir>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    _ui(new Ui::MainWindow)
{
    /// setup objects
    _ui->setupUi(this);
    _jsBridge   = new JsBridge(this);
    _serial     = new Serial(this, _jsBridge);

    /// remove scrollbars
    _ui->webView->page()->mainFrame()->setScrollBarPolicy(Qt::Horizontal, Qt::ScrollBarAlwaysOff);
    _ui->webView->page()->mainFrame()->setScrollBarPolicy(Qt::Vertical, Qt::ScrollBarAlwaysOff);

    /// Load webpage contents
    loadHtmlFromFile(QDir::currentPath()+"/map.html");
    setStatus("Loading map..");

    /// Declare object "qt" in javascript
    _ui->webView->page()->mainFrame()->addToJavaScriptWindowObject("qt", _jsBridge);

    /// connect to serial device
    if (_serial->connect("ttyUSB0",9600,static_cast<QSerialPort::DataBits>(8),static_cast<QSerialPort::Parity>(0),static_cast<QSerialPort::StopBits>(1),static_cast<QSerialPort::FlowControl>(0)))
    {
        setStatus("Connected to " + _serial->port());
    }
    else
    {
        setStatus("Unable to connect to ttyUSB0: "+_serial->error());
    }
}



MainWindow::~MainWindow()
{
    delete _ui;
}


/// Set status window in the lowest bar in main window
void MainWindow::setStatus(QString message)
{
    _ui->statusMessage->setText(message);
}


/// Send javascript code to webview and evaluate it
void MainWindow::evaluateJs(QString js)
{
    _ui->webView->page()->mainFrame()->evaluateJavaScript(js);
}


/// Loads a file to webview
void MainWindow::loadHtmlFromFile(QString _file)
{
    QFile file(_file);
    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QString html = QTextStream(&file).readAll();
        _ui->webView->setHtml(html);
    }
}


/// Button Reload = reload webview content
void MainWindow::on_pushButton_1_clicked()
{
    setStatus("Map cleared");
    _jsBridge->clear();
    loadHtmlFromFile(QDir::currentPath()+"/map.html");
    _ui->webView->page()->mainFrame()->addToJavaScriptWindowObject("qt", _jsBridge);
}


/// Button Upload = upload waypoints to connected UAVs
void MainWindow::on_pushButton_2_clicked()
{
    setStatus("Uploading to drones");
    _serial->write("WPC\n");  /// WPC = WayPoint Clear = Delete all waypoints

    QStringList js;
    js << "for (var i=0;i < _markers.length;i++) {";
    js << "qt.waypoint(_markers[i].getPosition().lat(), _markers[i].getPosition().lng(), 0.0);";
    js << "}";
    js << "qt.upload();";

    evaluateJs(js.join("\n"));
}


/// Button Quit = Quit program
void MainWindow::on_pushButton_3_clicked()
{
    exit(0);
}
