/*
Find red colors from an image and report run time
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during spring 2013

compile with: g++ -ggdb `pkg-config --cflags opencv` -o find_red find_red.cpp `pkg-config --libs opencv`
*/




#include <stdio.h>
#include <ctime>		/// time()
#include <opencv2/opencv.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/stitching/stitcher.hpp>

using namespace cv;


void findRed(cv::Mat& img, cv::Mat& out, int thresholdBlue = 100, int thresholdGreen = 100, int thresholdRed = 150)
{
	img.copyTo(out);
	
	inRange(img, Scalar(0, 0, thresholdRed), Scalar(thresholdBlue, thresholdGreen, 255), out);
	out = out < 128;
}


int _thresholdBlue = 50;
int _thresholdGreen = 50;
int _thresholdRed = 80;
int _dilate = 0;
int _erode = 3;


int main(int argc, char** argv)
{
	CvScalar s;
	clock_t start, stop;
	double t = 0.0;	
	
	if (argc < 2)
	{
		printf("Syntax: %s <in.img> <out.img>\n", argv[0]);
		return 1;
	}
	
	/// Image holders
	Mat frame = imread(argv[1]);
	Mat modified;
	Mat result;
	
	/// start internal clock
	printf("Clock started.\n");
	assert((start = clock())!=-1);

	/// find red and receive binary image
	findRed(frame, modified, _thresholdBlue, _thresholdGreen, _thresholdRed);	/// three channels
	
	/// Remove unwanted objects and enhance the rest
	dilate( modified, modified, cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*_dilate + 1, 2*_dilate+1 ), cv::Point( _dilate, _dilate ) ) );
	erode( modified, result, cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*_erode + 1, 2*_erode+1 ), cv::Point( _erode, _erode ) ) );

	stop = clock();
	t = (double) (stop-start)/CLOCKS_PER_SEC;
	printf("Run time: %fs\n", t);

	if (imwrite(argv[2],result))
	{
		printf("Saved\n");
	}


	return 0;
}
