/*
written by Vegard Hammerseth <vegard@hammerseth.com> (http://vegard.hammerseth.com) during spring 2013
*/


char * IDENTITY = "F#5";  /// To work with SARCC

///
/// NAVIGATION
///
#include "/home/v/Dropbox/Software/arduino/navigation_/TinyGPS.cpp"    /// GPL class for parsing NMEA written by ...not me...
TinyGPS _GPS;
float _latitude;
float _longitude;
float _altitude;
long unsigned int _age;

/// Setup serial ports when not using Teensy
#ifndef __MK20DX128__
#include <SoftwareSerial.h>
SoftwareSerial Serial1(2, 3);
SoftwareSerial Serial2(4, 5);
#endif



/// Make LED blink
void blink(int requestedBlinks = 1, int pause = 100)
{
  pinMode(13,OUTPUT);
  digitalWrite(13,LOW);
  int state = HIGH;
  delay(pause);
  for (int i=0;i<requestedBlinks;i++)
  {
    digitalWrite(13,state);
    state = (state==LOW?HIGH:LOW);
    delay(pause);
  }
  
  digitalWrite(13,LOW);
}


/// Update variables for current position
void updatePosition()
{
  if (!Serial1.available())
  {
    return;
  }
  
  int c = Serial1.read();
  //Serial.write(c);
    
  /// a NMEA line is ready to be parsed
  if (_GPS.encode(c))
  {
    /// new data available - parse position from NMEA and save to latitude and longitude
    _GPS.f_get_position(&_latitude, &_longitude, &_age);
    _altitude = _GPS.f_altitude();
  }
}


///
/// ----INITIATION----
///
void setup()
{
  Serial.begin(115200);
  while (!Serial)
  {
    blink();
  }

  
  ///
  /// CONNECT BASE STATION (WIRELESS TRANSCEIVER)
  ///
  Serial2.begin(9600);                              /// Serial2 means pin 9 and 10 on Teensy 3.0
  Serial2.println("------------------------------------RESTARTED----------------------"); 
 

  ///
  /// CONNECT GPS
  ///
  Serial1.begin(9600);
  Serial2.println("NAVIGATION: GPS serial started, waiting for satellites");
  
  
  /// wait until at least four satellites are connected then another five seconds such that GPS can iterate to a better position or even track more satellites
  unsigned long time = millis();
  int satellites     = 0;
  /*while ((millis()-time) <= 5000)  /// preferably we would have more than 4, but in cases where only 4 is available the vehicle would be useless
  {
    updatePosition();
    
    satellites = (_GPS.satellites() == TinyGPS::GPS_INVALID_SATELLITES ? 0 : _GPS.satellites());

    /// Check if enough satellites are connected
    if ( satellites < 4 && (millis()-time) >= 800)
    {
      Serial2.print(satellites);
      Serial2.println(" satellites");
      time = millis();
      blink(2,100);
    }
  }*/
  
}


///
/// MAIN LOOP TIMERS
///
unsigned long time;
unsigned long pHz01 = 0;
unsigned long pHz1 = 0;



///
/// CONTINUOUS LOOP (MAIN LOOP)
///
void loop()
{
  time = micros();   /// Store time in memory for this iteration. This will overflow in 70 minutes.
  
  updatePosition();  /// Update current position using GPS
  
  /// 0.1 Hz
  if ( (time - pHz01) >= 10000000)
  {
    hz01();
    pHz01 = time;
  }
  
  /// 1 Hz
  if ( (time - pHz1) >= 1000000)
  {
    hz1();
    pHz1 = time;
  }
  
  /// When received character "H" from host, send found-notification and coordinates over wireless
  if (Serial.read() == 'H')
  {
    Serial2.print("H ");
    Serial2.print(_latitude, 8);
    Serial2.print(" ");
    Serial2.print(_longitude, 8);
    Serial2.print(" ");
    Serial2.println(_altitude, 8);
    blink();
  }
}



///
/// 0.1 HZ LOOP
///
void hz01()
{
  Serial2.print(_GPS.satellites() == TinyGPS::GPS_INVALID_SATELLITES ? 0 : _GPS.satellites());
  Serial2.println(" satellites");
  /// Send vehicle position to base station
  Serial2.print(IDENTITY);
  Serial2.print(" P ");
  Serial2.print(_latitude, 8);
  //Serial2.print(63.100,8);
  Serial2.print(" ");
  Serial2.print(_longitude, 8);
  //Serial2.print(10.11,8);
  Serial2.print(" ");
  Serial2.println(_altitude, 8);
}



///
/// 1 HZ LOOP
///
void hz1()
{
  Serial.print(_latitude, 8);
  Serial.print(" ");
  Serial.print(_longitude, 8);
  Serial.print(" ");
  Serial.print(_altitude, 8);
  Serial.println(" ");
}
